
// Test fixture for IDT71V432 Synchronous SRAM
//

`timescale 1ns / 100ps

`define Max 16

module main;

parameter Tsa = 2.5;
parameter Tds = 2.5;
parameter Tdh = 0.5;
parameter Tkc = 15.0;
parameter Tkq = 4.5;

reg   [14:0] A;
reg          CLK;
reg          ADSP_;
reg          ADV_;
reg          LBO_;
reg          ADSC_;
reg    [4:1] BW_;
reg          BWE_;
reg          GW_;
reg          CE_;
reg          CS0;
reg          CS1_;
reg          OE_;

reg   [31:0] DataOut;
reg   [31:0] TempReg;

reg   [31:0] DQ;
wire  [31:0] DQbus = DQ;

reg   [31:0] RandomData[0:`Max-1];
reg   [31:0] BurstData[0:`Max-1];

integer   i, j,
          result;


initial begin
  $recordfile ("v432.sst",20);
  $recordvars;

  result = $fopen("v432.res"); if (result == 0) $finish;
end


always begin
  @(posedge CLK) 
    $fdisplay(result,
      "%b",  ADSC_,
      "%b",  ADSP_,
      "%b",  BWE_,
      "%b",  CE_,
      "%b",  CS0,
      "%b",  CS1_,
      "%b",  LBO_,
      "%b",  OE_,
      "%b",  BW_,  // 4 bits
      "%b",  ADV_,
      "%b ", GW_,

      "%h ", DQbus,

      "%h ", A,

      " ;%d", $stime
      );
end


initial begin
  ADSC_ = 1;
  ADSP_ = 1;
  BWE_  = 1;
  CE_   = 0;
  CS0   = 1;
  CS1_  = 0;
  LBO_  = 0;
  OE_   = 1;
  CLK   = 0;
  BW_   = 4'hf;
  ADV_  = 1;
  GW_   = 1;
  for (i=0;i<`Max;i=i+1) begin      // Generate random data for testing
    RandomData[i] = $random;
  end

  $display("Simple read/write test");
  for (i=0;i<`Max;i=i+1) begin      // Test straight write/read
    write(i, RandomData[i]);
    read(i, DataOut, RandomData[i]);
  end
 $display("CE_ disable");
 read(3, DataOut, RandomData[3]);
 disable_ce;
 read(7, DataOut, RandomData[7]);
 disable_cs0;
 read(2, DataOut, RandomData[2]);
  for (i=0;i<`Max;i=i+1) begin      // Fill RAM with zero's
    write(i, 0);
  end


  $display("Byte mode read/write test");
//  GW_ = 1;                        // Disable global write
//  BWE_  = 0;                      // Enable byte write
  for (i=0;i<`Max;i=i+1) begin    // Test byte write/read
    BW_ = $random;
    TempReg = RandomData[i];
    byte_write(i, TempReg);
    if ( BW_[1] == 1 ) TempReg[7:0] = 0;
    if ( BW_[2] == 1 ) TempReg[15:8] = 0;
    if ( BW_[3] == 1 ) TempReg[23:16] = 0;
    if ( BW_[4] == 1 ) TempReg[31:24] = 0;
    read(i, DataOut, TempReg);
  end
  BWE_  = 1;                      // Disable byte write

  // Test burst mode write/read
  $display("Burst mode read/write test");
  for (i=0;i<`Max;i=i+1) begin      // Test byte write/read
      BurstData[i] = RandomData[i];
  end

  GW_ = 0;                       // Enable global write
  for (i=0;i<`Max;i=i+4) begin   // Write data from BurstData buffer
    burst_write(i,4); 
  end
  GW_ = 1;                       // Disable global write

  for (i=0;j<`Max;i=i+1) begin   // Clear data buffer
      BurstData[i] = 0;
  end
  
  for (i=0;i<`Max;i=i+4) begin
    burst_read(i,4); 
    for (j=i;j<i+4;j=j+1) begin      // Test byte write/read
//      if ( BurstData[j] != RandomData[j] )
//          $display("%d  Burst error: Addr %h Exp %h Act %h", $stime, j, RandomData[j], BurstData[j]);
    end
  end
  $finish;
end

always #(Tkc/2) CLK = ~CLK;

idt71v432s7 dut (
    .A        (A),
    .D        (DQbus),
    .oe_      (OE_),
    .ce_      (CE_),
    .cs0      (CS0),
    .cs1_     (CS1_),
    .lbo_     (LBO_),
    .gw_      (GW_),
    .bwe_     (BWE_),
    .bw4_     (BW_[4]),
    .bw3_     (BW_[3]),
    .bw2_     (BW_[2]),
    .bw1_     (BW_[1]),
    .adsp_    (ADSP_),
    .adsc_    (ADSC_),
    .adv_     (ADV_),
    .clk      (CLK)
    );

task disable_ce;
begin
    OE_ = 0;
    if (CLK)
        @( negedge CLK );
    ADSC_ = 0;
    CE_   = 1;
    @( posedge CLK );
    @( negedge CLK );
    ADSC_ = 1;
    CE_   = 0;
end
endtask

task disable_cs0;
begin
    OE_ = 0;
    if (CLK)
        @( negedge CLK );
    ADSP_ = 0;
    CS0   = 0;
    @( posedge CLK );
    @( negedge CLK );
    ADSP_ = 1;
    CS0   = 1;
end
endtask

task read;
input  [14:0] addr;
output [31:0] data;
input  [31:0] exp;
begin
    if ( CLK )
        @( negedge CLK );
    DQ = 32'hzzzzzzzz;
    ADV_  = 1;
    A = addr;
    ADSP_ = 0;
    @( posedge CLK );      // SRAM latches Address and begins internal read
    @( negedge CLK );
    ADSP_ = 1;
    OE_   = 0;
    @( posedge CLK );      // SRAM begins placing data onto bus
    @( posedge CLK );      // Data sampled by reading device
//  #Tkq data = DQbus;
    data = DQbus;          // Hopefully the SRAM has an output hold time
    if ( data !== exp )
        $display("%d Read error: Addr %h Exp %h Act %h", $stime, addr, exp, data);
    @( negedge CLK );
    OE_   = 1;
end
endtask

task burst_read;
input  [14:0] addr;
input  [31:0] n;
integer       i;
begin
    DQ = 32'hzzzzzzzz;
    if ( CLK )
        @( negedge CLK );
    #1 A = addr;
    ADSP_ = 0;
    @( posedge CLK );           // Address latched by SRAM, begins internal read
    @( negedge CLK );
    ADSP_ = 1;
    #1 OE_   = 0;
    ADV_  = 0;
    @( posedge CLK );           // SRAM begins placing data onto bus
//    for (i=addr;i<addr+n;i=i+1) begin
//        @( posedge CLK );
//        #Tkq;
//        BurstData[i] = DQbus;
//    end
 
    for (i=addr;i<addr+n;i=i) begin
        @( posedge CLK );
        BurstData[i] = DQbus;
        @( negedge CLK );
        if ( ($random & 3) === 2'b11 ) // suspend burst 25% of the time
            ADV_ = 1;
        else begin
            ADV_ = 0;
            i=i+1;
        end
    end
 

    #1 OE_   = 1;
    ADV_  = 1;
end
endtask

task write;
input  [14:0] addr;
input  [31:0] data;
begin
    if ( CLK )
        @( negedge CLK );
    ADV_  = 1;
    A = addr;
    ADSP_ = 0;
    @( negedge CLK );
    ADSP_ = 1;
    GW_ = 0;
    #(Tkc/2-Tds) DQ = data;
    @( posedge CLK );
    @( negedge CLK );
    GW_ = 1;
    DQ = 32'hzzzzzzzz;
end
endtask

task byte_write;
input  [14:0] addr;
input  [31:0] data;
begin
    if ( CLK )
        @( negedge CLK );
    ADV_  = 1;
    A = addr;
    ADSP_ = 0;
    @( negedge CLK );
    ADSP_ = 1;
    BWE_ = 0;
    #(Tkc/2-Tds) DQ = data;
    @( posedge CLK );
    @( negedge CLK );
    BWE_ = 1;
    DQ = 32'hzzzzzzzz;
end
endtask

task burst_write;
input  [14:0] addr;
input  [31:0] n;
integer       i;
begin
    if ( CLK )
        @( negedge CLK );
    #1 A = addr;
    ADSP_ = 0;
    for (i=addr;i<addr+n;i=i+1) begin
        @( negedge CLK );
        ADSP_ = 1;
        if (addr!=i) ADV_  = 0;
        #(Tkc/2-Tds) DQ = BurstData[i];
        @( posedge CLK );
    end
    @( negedge CLK );
    ADV_  = 1;
end
endtask

endmodule

